import re

# Empirically guessed
xScale, yScale = 280.0, 333.3

# rightbarb could be used instead of leftarc in both of these
athenianBasics = """
    rightscroll doublescroll boathookandscroll leftbarb rightarc
    leftarc arcandscroll doubleleftbarb barbandarc doublebarb
    doublearc arcandbarb
""".split()

athenianDoubles = """
    doublescroll boathookandscroll leftbarb rightarc
    rightbarb arcandscroll doubleleftbarb barbandarc doublebarb
""".split()

athenianSet = athenianBasics + [x+"double" for x in athenianDoubles] + [
        x+"triple" for x in athenianBasics] + [x+"ex" for x in athenianDoubles]

lineexpr = 'C (\d{1,3}) ; WX \d{1,3} ; N ([a-z]+) ; B (-?\d{1,3}) (-?\d{1,3}) (-?\d{1,3}) (-?\d{1,3}) ;'
codes = re.findall(lineexpr, open('Sagittal.afm').read())
# Hack: ignore the first two
del codes[:2]

output = open('sagittal.ly', 'w')

output.write('''%% Support for using sagittal glyphs

#(define (sagittalString names)
    (apply string-append (map (lambda (name)
        (ly:wide-char->utf-8 (+ %i (cadr (assoc name sagittalMetrics)))))
        names)))

#(define (sagittalXExtent names)
    (if (null? names) '(0.0 . 0.0)
        (let ((current (caddr (assoc (car names) sagittalMetrics)))
              (others (sagittalXExtent (cdr names))))
        (cons
            (+ (car current) (car others))
            (+ (cdr current) (cdr others))))))

#(define (sagittalYExtent names)
    (if (null? names) '(0.0 . 0.0)
        (let ((current (cdddr (assoc (car names) sagittalMetrics)))
              (others (sagittalYExtent (cdr names))))
        (cons
            (+ (car current) (car others))
            (+ (cdr current) (cdr others))))))

pureAthenianStrings = #'(
    (0 natural)
''' % 0xF000)

for i, glyph in enumerate(athenianSet):
    iglyph = i+1, glyph
    output.write("    (%i %sup)\n" % iglyph)
    output.write("    (-%i %sdown)\n" % iglyph)

output.write(''')

mixedAthenianStrings = #'(
    (0 natural)
''')

# Intersection of pure and mixed:
for i, glyph in enumerate(athenianBasics):
    iglyph = i+1, glyph
    output.write("    (%i %sup)\n" % iglyph)
    output.write("    (-%i %sdown)\n" % iglyph)

# lowering of sharps/raising of flats:
for i, glyph in enumerate(reversed(athenianBasics[:8])):
    iglyph = i+1 + len(athenianBasics), glyph
    output.write("    (%i %sdown sharp)\n" % iglyph)
    output.write("    (-%i %sup flat)\n" % iglyph)

sharpDegree = iglyph[0] + 1

output.write("    (%i sharp)\n" % sharpDegree)
output.write("    (-%i flat)\n" % sharpDegree)

# raising of sharps/lowering of flats:
for i, glyph in enumerate(athenianBasics):
    iglyph = i+1 + sharpDegree, glyph
    output.write("    (%i %sup sharp)\n" % iglyph)
    output.write("    (-%i %sdown flat)\n" % iglyph)

# lowering of double sharps/raising of double flats:
for i, glyph in enumerate(athenianBasics[:8]):
    iglyph = 2*sharpDegree - i-1, glyph
    output.write("    (%i %sdown doublesharp)\n" % iglyph)
    output.write("    (-%i %sup doubleflat)\n" % iglyph)

output.write("    (%i doublesharp)\n" % (2*sharpDegree))
output.write("    (-%i doubleflat)\n" % (2*sharpDegree))

# raising of double sharps/lowering of double flats:
for i, glyph in enumerate(athenianBasics):
    iglyph = i+1 + 2*sharpDegree, glyph
    output.write("    (%i %sup doublesharp)\n" % iglyph)
    output.write("    (-%i %sdown doubleflat)\n" % iglyph)


output.write(''')

sagittalMetrics = #'(
''')

for code, name, x0, y0, x1, y1 in codes:
    output.write("    (%s %i (%.3f . %.3f) %.3f . %.3f)\n" %(
        name, int(code),
        float(x0)/xScale, float(x1)/xScale,
        float(y0)/yScale, float(y1)/yScale))

output.write(''')

% Allow the Sagittal font to come from the current directory
#(ly:font-config-add-directory ".")
''')

"""
Examples of metrics and known extents:
    ( 0  1 (0 . 0.6) (-1.1 . 0.5) rightarcup) ; 2/72
C 156 ; WX 218 ; N rightarcup ; B 0 -360 157 172 ;
    ( 0 -1 (0 . 0.6) (-0.5 . 1.1) rightarcdown) ; 2/72
C 132 ; WX 218 ; N rightarcdown ; B 0 -172 157 360 ;
    ( 0  0 (0 . 0.7) (-1.1 . 1.1) natural)
C 144 ; WX 250 ; N natural ; B 0 -360 188 360 ;

188/157 = 1.20
0.7/0.6 = 1.17
probably close enough for the third number to be a scaled X extent
The second and fourth numbers seem to be Y extents
Standardize on 1.1 -> 360 or 1 -> 360?

360/1.1 = 327.3
172/0.5 = 344.0
188/0.7 = 268.6
157/0.6 = 261.7
Looks like X and Y extents are scaled differently.
Naturals are probably not reliable because they're an odd shape
"""
