\include "tripod-data.ly"

majorScale = #'#(0 1 2 5/2 7/2 9/2 11/2)
#(ly:set-default-scale (ly:make-scale majorScale))

#(define (trojanAlteration degree pitch)
     (let* ((scaleSize (vector-length majorScale))
            (modDegree (modulo degree scaleSize))
            (octave (/ (- degree modDegree) scaleSize)))
        (- pitch (+ (* 6 octave) (vector-ref majorScale modDegree)))))

% Semitoes can nudge the degree, hence the fractional degrees.
%                             B    C Db  D#  E F Gb  G#  A
defaultDiatonicDegrees = #'#(-1    0 0.8 1.2 2 3 3.8 4.2 5)
degreesWithSharps =      #'#(-1.2 -1 0   1   2 2 3   4   5)
degreesWithFlats =       #'#( 0    0 1   2   3 3 4   5   4)

#(define (trojanFromTripod tripodPitch ET diatonicDegrees)
    (let* ((sharpSize (best-sharp ET))
           (semitoeSize (best-semitoe ET))
           (inchSize (best-inch ET))
           (commaSize (best-comma ET))
           (largestAlteration (apply max (map car pureAlterations)))
           (tripodDegree (car tripodPitch))
           (semitoes (cadr tripodPitch))
           (sharps (caddr tripodPitch))
           (inches (cadddr tripodPitch))
           (commas (car (cddddr tripodPitch)))
           (tripodScale (tripodNominals (+ sharpSize semitoeSize) inchSize))
           (logicalDegree (nint (+ (/ semitoes 3)
                   (vector-ref diatonicDegrees (- tripodDegree 1)))))
           (pitch (- (+ (* sharps sharpSize) (* semitoes semitoeSize)
                        (* inches inchSize) (* commas commaSize)
                        (vector-ref tripodScale (- tripodDegree 1)))
                     (vector-ref tripodScale 1))) ; tuning of C
           (alteration (trojanAlteration logicalDegree pitch))
           (trojanDegree (if (< alteration (- largestAlteration)) (- logicalDegree 1)
                             (if (> alteration largestAlteration) (+ logicalDegree 1)
                                 logicalDegree))))
         (cons trojanDegree (trojanAlteration trojanDegree pitch))))

#(define (trojanName tripodName ET diatonicDegrees suffix)
    (let* ((name (symbol-append (car tripodName) suffix))
           (trojanPitch
                (trojanFromTripod (cdr tripodName) ET diatonicDegrees))
           (scaleDegree (car trojanPitch))
           (alteration (cdr trojanPitch))
        )
        (cons name
            (if (< scaleDegree 0) (ly:make-pitch -2 (+ scaleDegree 7) alteration)
                (if (> scaleDegree 6) (ly:make-pitch 0 (- scaleDegree 7) alteration)
                    (ly:make-pitch -1 scaleDegree alteration))))))

trojanPitchNames = #(append
    (map (lambda (name)
        (trojanName name tuning defaultDiatonicDegrees #f)) tripodNames)
    (map (lambda (name)
        (trojanName name tuning degreesWithSharps 'ms)) tripodNames)
    (map (lambda (name)
        (trojanName name tuning degreesWithFlats 'mf)) tripodNames)
)

mixedStrings = #(ratio-strings mixedAlterations)
pureStrings = #(ratio-strings pureAlterations)

#(ly:parser-set-note-names parser trojanPitchNames)

mixed = {
    \override Accidental #'stencil = #ly:text-interface::print
    \override Accidental #'font-name = #"Sagittal"
    \override Accidental #'font-size = #4
    \override Accidental #'text = #(lambda (grob)
        (cadr (assoc (ly:grob-property grob 'alteration) mixedStrings)))
    \override Accidental #'X-extent = #(lambda (grob)
        (caddr (assoc (ly:grob-property grob 'alteration) mixedStrings)))
    \override Accidental #'Y-extent = #(lambda (grob)
        (cadddr (assoc (ly:grob-property grob 'alteration) mixedStrings)))
}

pure = {
    \override Accidental #'stencil = #ly:text-interface::print
    \override Accidental #'font-name = #"Sagittal"
    \override Accidental #'font-size = #4
    \override Accidental #'text = #(lambda (grob)
        (cadr (assoc (ly:grob-property grob 'alteration) pureStrings)))
    \override Accidental #'X-extent = #(lambda (grob)
        (caddr (assoc (ly:grob-property grob 'alteration) pureStrings)))
    \override Accidental #'Y-extent = #(lambda (grob)
        (cadddr (assoc (ly:grob-property grob 'alteration) pureStrings)))
}

% Only 60 and 72 supported:
fetaGlyphs = #(if (= tuning 60) fifthGlyphs sixthGlyphs)

\layout {
  \context {
    \Score
    \override Accidental #'stencil = #ly:text-interface::print
    \override Accidental #'font-name = #"Sagittal"
    \override Accidental #'font-size = #4
    \override Accidental #'text = #(lambda (grob)
        (cadr (assoc (ly:grob-property grob 'alteration) mixedStrings)))
    \override Accidental #'X-extent = #(lambda (grob)
        (caddr (assoc (ly:grob-property grob 'alteration) mixedStrings)))
    \override Accidental #'Y-extent = #(lambda (grob)
        (cadddr (assoc (ly:grob-property grob 'alteration) mixedStrings)))
  }
}

\layout {
  \context {
    \Score
    \override Accidental #'glyph-name-alist = \fetaGlyphs
    \override KeySignature #'glyph-name-alist = \fetaGlyphs
    \override AccidentalCautionary #'glyph-name-alist = \fetaGlyphs
    \override TrillPitchAccidental #'glyph-name-alist = \fetaGlyphs
    \override AmbitusAccidental #'glyph-name-alist = \fetaGlyphs
  }
}

\paper{
    newStaffSize = #22
    #(define fonts (make-pango-font-tree "URW Bookman L" "Sans" "Mono"
                    (/ newStaffSize 20)))
}
lyricsize=\tiny

% Tripod note names in a given register.
tripod = #(define-music-function (parser location register m)
  (integer? ly:music?)
  (ly:music-transpose m (ly:make-pitch (- register 3) 0 0)))

fiveclef = {\clef treble \mixed}
fourfiveclef = {\clef treble \mixed}
fourclef = {\clef treble \mixed}
threeclef = {\clef bass \mixed}

